from django.db.models.query_utils import Q
from samklang_utils.managers import PermissionManager

from datetime import datetime

class LiveEntryManager(PermissionManager):
    """Manager that returns posts which is set to be shown live"""

    def get_query_set(self):
        now = datetime.now
        return super(LiveEntryManager, self).get_query_set().filter(pub_date__lt=now).filter(
                Q(pub_enddate=None) | Q(pub_enddate__gt=now))



from django.template.loader import render_to_string
from samklang_pages.pagewidgets import Widget
from samklang_blog.models import Entry

class Latest(Widget):
    """Widget listing latest entries"""

    def render(self):
        limit = self.options.get('limit', 5)
        entries = Entry.live.all()[:limit]

        return render_to_string(
            'samklang_blog/latest_entries.html',
            {'object_list': entries,}
        )

class BlogFront(Widget):
    """For inclusion in main page space"""

    def render(self, request):
        limit = self.options.get('limit', 6)
        entries = Entry.live.all()[:limit]

        return render_to_string(
            'samklang_blog/blog.html',
            {'object_list': entries,}
        )




from django.conf.urls.defaults import *
from samklang_blog.views import EntryCreateView, EntryUpdateView, EntryArchiveIndexView
from samklang_blog.views import EntryYearArchiveView, EntryMonthArchiveView, EntryDateDetailView

urlpatterns = patterns('',
    url(r'^new/$', EntryCreateView.as_view(), name='blog-entry-new'),
    url(r'^edit/(?P<pk>\d+)/$', EntryUpdateView.as_view(), name='blog-entry-edit'),
    url(r'^(?P<year>\d{4})/(?P<month>\d{2})/(?P<day>\d{2})/(?P<slug>[-\w]+)/$', EntryDateDetailView.as_view(), name='blog-entry-detail'),
    url(r'^(?P<year>\d{4})/(?P<month>\d{2})/$', EntryMonthArchiveView.as_view(), name='blog-entry-archive-month'),
    url(r'^(?P<year>\d{4})/$', EntryYearArchiveView.as_view(), name='blog-entry-archive-year'),
    url(r'^$', EntryArchiveIndexView.as_view(), name='blog-entry-archive-index'),
)



from django.contrib import admin
from samklang_blog.models import Entry

class EntryAdmin(admin.ModelAdmin):
    prepopulated_fields = { 'slug': ['title'] }
    fields = ['title', 'slug', 'body', 'user', 'group', 'site', 'pub_date', 'pub_enddate',]# 'tag_list']
    list_display = ('title', 'user', 'group', 'pub_date', 'updated_date')
    list_filter = ('group', 'pub_date')
    ordering = ('-pub_date',)
    search_fields = ('title', 'body_html')

    def queryset(self, request):
        if request.user.is_superuser:
            qs = self.model.objects.get_query_set()
        else:
            qs = self.model.objects.get_query_set().filter(author = request.user)

        return qs
admin.site.register(Entry, EntryAdmin)



from django.forms import ModelForm
from samklang_blog.models import Entry
from samklang_utils.forms import MarkdownTextarea, AutoupdateTextInput#, DateTimePicker

class EntryForm(ModelForm):

    class Meta:
        model = Entry
        fields = ('title', 'body', 'group', 'pub_date', 'pub_enddate')
        widgets = {
                'body': MarkdownTextarea(),
                'title': AutoupdateTextInput(),
                #'pub_date': DateTimePicker(),
                #'pub_enddate': DateTimePicker(),
                }





from django import template
from samklang_blog.models import Entry

register = template.Library()

@register.simple_tag(takes_context=True)
def latest_entries(context, number):
    """get a list, latest, of the 'numbember' latest entries"""
    latest = Entry.live.all()[:number]
    context['latest_entries'] = latest
    return ''



from django.http import HttpResponseRedirect
from django.views.generic.edit import CreateView, UpdateView
from django.views.generic.dates import ArchiveIndexView, YearArchiveView, MonthArchiveView, DateDetailView
from django.contrib.auth.decorators import login_required
from django.utils.decorators import method_decorator

from django.contrib.sites.models import Site

from samklang_blog.models import Entry
from samklang_blog.forms import EntryForm

from datetime import datetime

MONTH_FORMAT = '%m'

class EntryCreateView(CreateView):
    model = Entry
    form_class = EntryForm
    initial = {'pub_date': datetime.now().strftime("%Y-%m-%d %H:%M:%S")}
    month_format = MONTH_FORMAT

    def form_valid(self, form):
        self.object = form.save(commit=False)
        self.object.user = self.request.user
        if hasattr(self.request, 'site'):
            self.object.site = self.request.site
        else:
            self.object.site = Site.objects.get(pk=1)
        self.object.save()
        return HttpResponseRedirect(self.object.get_absolute_url())

    @method_decorator(login_required)
    def dispatch(self, *args, **kwargs):
        return super(EntryCreateView, self).dispatch(*args, **kwargs)

class EntryUpdateView(UpdateView):
    model = Entry
    form_class = EntryForm
    month_format = MONTH_FORMAT

    #def form_valid(self, form):
    #    self.object = form.save()
    #    return HttpResponseRedirect(self.object.get_absolute_url())

    @method_decorator(login_required)
    def dispatch(self, *args, **kwargs):
        return super(EntryUpdateView, self).dispatch(*args, **kwargs)

class EntryArchiveIndexView(ArchiveIndexView):
    model = Entry
    date_field = 'pub_date'
    month_format = MONTH_FORMAT
    allow_empty = True

    def get_queryset(self):
        return Entry.live.all()

class EntryYearArchiveView(YearArchiveView):
    model = Entry
    date_field = 'pub_date'
    month_format = MONTH_FORMAT
    allow_empty = True

class EntryMonthArchiveView(MonthArchiveView):
    model = Entry
    date_field = 'pub_date'
    month_format = MONTH_FORMAT
    allow_empty = True

class EntryDateDetailView(DateDetailView):
    model = Entry
    date_field = 'pub_date'
    month_format = MONTH_FORMAT



from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.models import User, Group
from django.contrib.sites.models import Site
from taggit.managers import TaggableManager

from samklang_utils import markdown, slugify
from samklang_utils.managers import PermissionManager
from samklang_blog.managers import LiveEntryManager

class Entry(models.Model):
    """
    Blog Entry.
    Specify which image class to use in settings.
    Example:
        BLOG_IMAGE_MODEL = { 'app': 'blog', 'model': 'image' }
    """

    # core fields
    title = models.CharField(_("title"), max_length=80, help_text=_("Maximum 80 characters."))
    body = models.TextField(_("contents"), help_text=_("Content written in markdown syntax."))
    pub_date = models.DateTimeField(blank=True, null=True, verbose_name=_("publish date"), help_text=_("Date from which the entry is shown live. Blank = draft."))
    pub_enddate = models.DateTimeField(blank=True, null=True, verbose_name=_("withdrawn date"), help_text=_("Date from which the entry is no longer accessible. Blank = live forever."))
    updated_date = models.DateTimeField(auto_now=True)

	# fields to store generated html
    body_html = models.TextField(editable=False, blank=True)

	# metadata
    site = models.ForeignKey(Site, verbose_name=_('site'))
    user = models.ForeignKey(User, verbose_name=_('user'))
    group = models.ForeignKey(Group, verbose_name=_('created for'), null=True, blank=True)
    slug = models.SlugField(unique_for_date='pub_date', help_text=_("Suggested value generated from title. Must be unique."))
    tags = TaggableManager(_('tags'), blank=True, help_text=_('A comma-separated list of tags.'))

    # managers, first one is default
    objects = PermissionManager()
    live = LiveEntryManager()

    class Meta:
        ordering = ['-pub_date']
        verbose_name_plural = _("entries")
        db_table = 'samklang_blog_entry'

    def save(self, *args, **kwargs):
        # convert markdown to html and store it
        self.body_html = markdown(self.body)
        self.slug = slugify(self.title)
        super(Entry, self).save(*args, **kwargs)

    def __unicode__(self):
        return u"%s, %s" % (self.title, self.body[0:50])

    @models.permalink
    def get_absolute_url(self):
        return ('blog-entry-detail', (), {
            'year': self.pub_date.strftime("%Y"),
            'month': self.pub_date.strftime("%m"),
            'day': self.pub_date.strftime("%d"),
            'slug': self.slug
        })



"""
This file demonstrates two different styles of tests (one doctest and one
unittest). These will both pass when you run "manage.py test".

Replace these with more appropriate tests for your application.
"""

from django.test import TestCase
from samklang_blog.models import Entry, Category
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType

from datetime import datetime

class EntryTest(TestCase):
    def test_live_entry(self):
        u = User()
        u.username = "testuser"
        u.save()
        c = Category()
        c.slug = "cat"
        c.title = "Cat"
        c.save()
        e = Entry()
        e.title = "test"
        e.slug = "test"
        e.body = "jadda"
        e.category = c
        e.user = u
        e.pub_date = datetime.now()
        e.save()

        f = Entry.live.get(title="test")
        self.assertEqual(e, f)




#!/usr/bin/env python
#coding: utf8

from distutils.core import setup

setup(
        name='samklang-blog',
        version="0.3.0",
        author='Jørgen Bergquist',
        author_email='gismos@gmail.com',
        url='http://github.com/sigurdga/samklang-blog',
        description='News and blog app for Samklang',
        long_description=open('README.txt').read(),
        license="AGPL",
        packages = ['samklang_blog', 'samklang_blog.templatetags', 'samklang_blog.migrations'],
        package_data = {'samklang_blog': ['templates/samklang_blog/*.html', 'locale/*/LC_MESSAGES/django.*o']},
        classifiers=[
                "Development Status :: 3 - Alpha",
                "License :: OSI Approved :: GNU Affero General Public License v3",
                "Intended Audience :: Developers",
                "Framework :: Django",
                "Environment :: Web Environment",
                "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
                ]
        )

